import hashlib
import json
import socket
import yaml
import requests

from enum import Enum
from itertools import cycle


class RunCode(Enum):
    CONTINUOUS = 0
    ONCE = 1
    UNTIL_SUCCESS = 2


class Account:

    def __init__(self, account_id, token, key, hq='http://localhost:8080'):
        self.hq = hq
        self.key = key
        self.headers = dict(accountID=account_id, token=token, key=hashlib.md5(str(key).encode('utf8')).hexdigest())

    def details(self):
        """ Get all account details """
        res = requests.get('%s/account' % self.hq, headers=self.headers)
        return res.json() if res.status_code == 200 else dict()

    def update_account(self, key: str, value: str):
        """ Update an account property """
        if key not in ['sleep', 'token', 'outpost_address', 'outpost_token']:
            raise KeyError('Property not recognized')
        requests.put('%s/account' % self.hq, headers=self.headers, json={key: value})

    def register_endpoint(self, identifier=socket.gethostname()):
        """ Register a new endpoint to your account """
        payload = dict(identifiers=[identifier])
        res = requests.post('%s/account/endpoint' % self.hq, headers=self.headers, json=payload)
        if res.status_code == 200:
            return res.json().get(identifier)
        raise Exception('Failed to register endpoint: %s' % res.text)

    def get_ttp(self, identifier=None):
        """ Get TTPs matching your filters """
        res = requests.get('%s/account/ttp' % self.hq, headers=self.headers, params=dict(id=identifier))
        if res.status_code == 200:
            return json.loads(self._xor(res.text))
        raise Exception('Failed to get procedure: %s' % res.text)

    def upload_ttp(self, file_path: str):
        """ Save a TTP on your Outpost server """
        with open(file_path, 'r') as stream:
            procedure = None
            if file_path.endswith('.yml'):
                procedure = yaml.safe_load(stream)
            procedure = procedure if procedure else json.loads(stream.read())

            encrypted_ttp = self._xor(json.dumps(procedure))
            res = requests.post('%s/account/ttp' % self.hq, headers=self.headers, data=encrypted_ttp)
            if res.status_code != 200:
                raise Exception('Failed to upload: %s' % res.text)

    def activate_ttp(self, identifier: str, run_code=RunCode.CONTINUOUS):
        """ Activate a TTP so endpoints will start running it """
        ttp = self.get_ttp(identifier)
        encrypted = self._xor(json.dumps(ttp, sort_keys=True, indent=2))
        checksum = hashlib.sha1(encrypted.encode('utf8')).hexdigest()
        res = requests.post(
            url='%s/account/activation/%s' % (self.hq, identifier),
            headers=self.headers,
            json=dict(checksum=checksum, run_code=run_code.value)
        )
        if res.status_code == 200:
            return dict(checksum=res.text)
        raise Exception('Failed to activate: %s' % res.text)

    def deactivate_ttp(self, identifier: str):
        """ Deactivate a TTP so endpoints will stop running it """
        res = requests.delete('%s/account/activation/%s' % (self.hq, identifier), headers=self.headers)
        if res.status_code != 200:
            raise Exception('Failed to deactivate: %s' % res.text)

    def account_summary(self, token=None):
        """ Get a summary of Account activity, or an individual Endpoint """
        res = requests.get('%s/account/results' % self.hq, headers=self.headers, params=dict(token=token))
        if res.status_code == 200:
            return res.json()
        raise Exception('Failed to get procedure: %s' % res.text)

    """ PRIVATE """

    def _xor(self, msg):
        return ''.join(chr(ord(c) ^ ord(k)) for c, k in zip(msg, cycle(self.key)))
